@@ -224,13 +224,6 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     # daemonize:
     starting_dir = os.getcwd()
 
-    clobber = upgrading or opts.use_display
-    try:
-        sockpath = dotxpra.server_socket_path(display_name, clobber)
-    except ServerSockInUse:
-        parser.error("You already have an xpra server running at %s\n"
-                     "  (did you want 'xpra upgrade'?)"
-                     % (display_name,))
     # Daemonize:
     if opts.daemon:
         logpath = dotxpra.conf_path(display_name) + ".log"
@@ -274,6 +267,41 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     scriptfile.write(xpra_runner_shell_script(xpra_file, starting_dir, opts.sockdir))
     scriptfile.close()
 
+    # Initialize the sockets before the display,
+    # That way, errors won't make us kill the Xvfb
+    # (which may not be ours to kill at that point)
+    sockets = []
+    if opts.bind_tcp:
+        try:
+            tcp_socket = create_tcp_socket(parser, opts.bind_tcp)
+            sockets.append(tcp_socket)
+            def cleanup_tcp_socket():
+                print("closing tcp socket")
+                try:
+                    tcp_socket.close()
+                except:
+                    pass
+            _cleanups.append(cleanup_tcp_socket)
+        except Exception, e:
+            print("cannot start - failed to create tcp socket at %s: %s" % (opts.bind_tcp, e))
+            return  1
+    #print("creating server socket %s" % sockpath)
+    clobber = upgrading or opts.use_display
+    try:
+        sockpath = dotxpra.server_socket_path(display_name, clobber)
+    except ServerSockInUse:
+        parser.error("You already have an xpra server running at %s\n"
+                     "  (did you want 'xpra upgrade'?)"
+                     % (display_name,))
+    sockets.append(create_unix_domain_socket(sockpath, opts.mmap_group))
+    def cleanup_socket():
+        print("removing socket %s" % sockpath)
+        try:
+            os.unlink(sockpath)
+        except:
+            pass
+    _cleanups.append(cleanup_socket)
+
     # Do this after writing out the shell script:
     os.environ["DISPLAY"] = display_name
 
@@ -300,6 +328,23 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         except OSError, e:
             sys.stderr.write("Error running xauth: %s\n" % e)
 
+    def xvfb_error(instance_exists=False):
+        if clobber:
+            return False
+        if xvfb.poll() is None:
+            return False
+        sys.stderr.write("\n")
+        sys.stderr.write("Xvfb command has terminated! xpra cannot continue\n")
+        sys.stderr.write("\n")
+        if instance_exists:
+            sys.stderr.write("There is an X11 server already running on display %s:\n" % display_name)
+            sys.stderr.write("You may want to use:\n" % display_name)
+            sys.stderr.write("  'xpra upgrade' if an instance of xpra is still connected to it\n")
+            sys.stderr.write("  'xpra --use-display start' to connect xpra to an existing X11 server only\n")
+        return True
+
+    if xvfb_error():
+        return  1
     # Whether we spawned our server or not, it is now running -- or at least
     # starting.  First wait for it to start up:
     try:
@@ -307,6 +352,8 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     except Exception, e:
         sys.stderr.write("%s\n" % e)
         return  1
+    if xvfb_error(True):
+        return  1
     # Now we can safely load gtk and connect:
     assert "gtk" not in sys.modules
     import gtk
@@ -320,10 +367,13 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     if clobber:
         xvfb_pid = get_pid()
     else:
+        if xvfb_error(True):
+            return  1
         xvfb_pid = xvfb.pid
 
     def kill_xvfb():
         # Close our display(s) first, so the server dying won't kill us.
+        print("killing xvfb with pid %s" % xvfb_pid)
         for display in gtk.gdk.display_manager_get().list_displays():
             display.close()
         if xvfb_pid is not None:
@@ -337,33 +387,6 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
     if not can_run_server():
         return  1
 
-    sockets = []
-    #print("creating server socket %s" % sockpath)
-    sockets.append(create_unix_domain_socket(sockpath, opts.mmap_group))
-    def cleanup_socket():
-        print("removing socket %s" % sockpath)
-        try:
-            os.unlink(sockpath)
-        except:
-            pass
-    _cleanups.append(cleanup_socket)
-    tcp_socket = None
-    if opts.bind_tcp:
-        try:
-            tcp_socket = create_tcp_socket(parser, opts.bind_tcp)
-            sockets.append(tcp_socket)
-        except Exception, e:
-            print("cannot start - failed to create tcp socket: %s" % e)
-            return  1
-    def cleanup_tcp_socket():
-        if tcp_socket:
-            print("closing tcp socket")
-            try:
-                tcp_socket.close()
-            except:
-                pass
-    _cleanups.append(cleanup_tcp_socket)
-
     # This import is delayed because the module depends on gtk:
     app = XpraServer(clobber, sockets, opts)
 
@@ -390,7 +413,7 @@ def run_server(parser, opts, mode, xpra_file, extra_args):
         return False # Only call once
     gobject.timeout_add(0, check_once)
 
-    _cleanups.append(app.cleanup)
+    _cleanups.insert(0, app.cleanup)
     if app.run():
         print("upgrading: not cleaning up Xvfb or socket")
         # Upgrading, so leave X server running
